@@ -1,12 +1,15 @@
 import json
+from copy import deepcopy
 
 import luigi
 from betterboto import client as betterboto_client
 
+from servicecatalog_puppet import manifest_utils_for_launches
+from servicecatalog_puppet import manifest_utils_for_spoke_local_portfolios
 from servicecatalog_puppet import aws
 from servicecatalog_puppet import config
 
-from servicecatalog_puppet import constants
+from servicecatalog_puppet import constants, manifest_utils
 from servicecatalog_puppet.workflow import tasks
 from servicecatalog_puppet.workflow import portfoliomanagement
 
@@ -54,10 +57,10 @@ class ProvisioningArtifactParametersTask(tasks.PuppetTask):
         with self.input().get('details').open('r') as f:
             details = json.loads(f.read())
             with betterboto_client.CrossAccountClientContextManager(
-                'servicecatalog',
-                f"arn:aws:iam::{self.account_id}:role/servicecatalog-puppet/PuppetRole",
-                f"{self.account_id}-{self.region}-sc",
-                region_name=self.region,
+                    'servicecatalog',
+                    f"arn:aws:iam::{self.account_id}:role/servicecatalog-puppet/PuppetRole",
+                    f"{self.account_id}-{self.region}-sc",
+                    region_name=self.region,
             ) as service_catalog:
                 logger.info(f"{self.uid}: getting path for {details.get('product_id')} of portfolio: {self.portfolio}")
                 path_id = aws.get_path_for_product(service_catalog, details.get('product_id'), self.portfolio)
@@ -86,8 +89,6 @@ class ProvisionProductTask(tasks.PuppetTask):
     manifest_parameters = luigi.DictParameter(default={}, significant=False)
     account_parameters = luigi.DictParameter(default={}, significant=False)
 
-    dependencies = luigi.ListParameter(default=[], significant=False)
-
     retry_count = luigi.IntParameter(default=1, significant=False)
     worker_timeout = luigi.IntParameter(default=0, significant=False)
     ssm_param_outputs = luigi.ListParameter(default=[], significant=False)
@@ -95,9 +96,6 @@ class ProvisionProductTask(tasks.PuppetTask):
     should_use_product_plans = luigi.BoolParameter(significant=False, default=False)
     requested_priority = luigi.IntParameter(significant=False, default=0)
 
-    pre_actions = luigi.ListParameter(default=[], significant=False)
-    post_actions = luigi.ListParameter(default=[], significant=False)
-
     try_count = 1
     all_params = []
 
@@ -134,7 +132,6 @@ class ProvisionProductTask(tasks.PuppetTask):
                 )
         self.all_params = all_params
 
-        dependencies = []
         version_id = portfoliomanagement.GetVersionIdByVersionName(
             self.portfolio,
             self.product,
@@ -148,22 +145,8 @@ class ProvisionProductTask(tasks.PuppetTask):
             self.account_id,
             self.region,
         )
-        for r in self.dependencies:
-            if r.get('status') is not None:
-                if r.get('status') == constants.TERMINATED:
-                    raise Exception("Unsupported")
-                new_r = r.get_wrapped()
-                del new_r['status']
-                dependencies.append(
-                    self.__class__(**new_r)
-                )
-            else:
-                dependencies.append(
-                    self.__class__(**r)
-                )
 
         return {
-            'dependencies': dependencies,
             'ssm_params': ssm_params,
             'version': version_id,
             'product': product_id,
@@ -174,7 +157,6 @@ class ProvisionProductTask(tasks.PuppetTask):
                 self.account_id,
                 self.region,
             ),
-            'pre_actions': [portfoliomanagement.ProvisionActionTask(**p) for p in self.pre_actions],
         }
 
     def api_calls_used(self):
@@ -349,9 +331,6 @@ class ProvisionProductTask(tasks.PuppetTask):
                                 f"[{self.uid}] Could not find match for {ssm_param_output.get('stack_output')}"
                             )
 
-                for p in self.post_actions:
-                    yield portfoliomanagement.ProvisionActionTask(**p)
-
                 with self.output().open('w') as f:
                     f.write(
                         json.dumps(
@@ -525,7 +504,7 @@ class TerminateProductTask(tasks.PuppetTask):
 
     parameters = luigi.ListParameter(default=[])
     ssm_param_inputs = luigi.ListParameter(default=[])
-    dependencies = luigi.ListParameter(default=[])
+    # dependencies = luigi.ListParameter(default=[])
 
     def requires(self):
         product_id = portfoliomanagement.GetProductIdByProductName(
@@ -621,11 +600,11 @@ class TerminateProductDryRunTask(tasks.PuppetTask):
 
     worker_timeout = luigi.IntParameter(default=0, significant=False)
 
-    try_count = 1
-
     parameters = luigi.ListParameter(default=[])
     ssm_param_inputs = luigi.ListParameter(default=[])
-    dependencies = luigi.ListParameter(default=[])
+    # dependencies = luigi.ListParameter(default=[])
+
+    try_count = 1
 
     def requires(self):
         product_id = portfoliomanagement.GetProductIdByProductName(
@@ -716,7 +695,6 @@ class ResetProvisionedProductOwnerTask(tasks.PuppetTask):
             "region": self.region,
         }
 
-
     def api_calls_used(self):
         return [
             f"servicecatalog.search_provisioned_products_{self.account_id}_{self.region}",
@@ -756,3 +734,232 @@ class ResetProvisionedProductOwnerTask(tasks.PuppetTask):
                         'OWNER': f"arn:aws:iam::{self.account_id}:role/servicecatalog-puppet/PuppetRole"
                     }
                 )
+
+
+class LaunchTask(tasks.PuppetTask):
+    launch_name = luigi.Parameter()
+    manifest = luigi.DictParameter()
+    puppet_account_id = luigi.Parameter()
+    should_use_sns = luigi.BoolParameter()
+    should_use_product_plans = luigi.BoolParameter()
+    include_expanded_from = luigi.BoolParameter()
+    single_account = luigi.Parameter()
+    is_dry_run = luigi.BoolParameter()
+
+    def params_for_results_display(self):
+        return {
+            "launch_name": self.launch_name,
+        }
+
+    def requires(self):
+        launch = self.manifest.get('launches').get(self.launch_name)
+
+        return {
+            'other_launches': [
+                self.__class__(
+                    launch_name=dependency,
+                    manifest=self.manifest,
+                    puppet_account_id=self.puppet_account_id,
+                    should_use_sns=self.should_use_sns,
+                    should_use_product_plans=self.should_use_product_plans,
+                    include_expanded_from=self.include_expanded_from,
+                    single_account=self.single_account,
+                    is_dry_run=self.is_dry_run,
+                ) for dependency in launch.get('depends_on', [])
+            ]
+        }
+
+    def generate_provisions(self, task_defs):
+        provisions = []
+
+        # task_defs = launch_task_def.get('task_defs')
+        for task_def in task_defs:
+            if self.single_account is not None:
+                if task_def.get('account_id') != self.single_account:
+                    continue
+
+            task_status = task_def.get('status')
+            del task_def['status']
+            del task_def['depends_on']
+            task_def['is_dry_run'] = self.is_dry_run
+
+            if task_status == constants.PROVISIONED:
+                provisioning_parameters = {}
+                for p in ProvisionProductTask.get_param_names(include_significant=True):
+                    provisioning_parameters[p] = task_def.get(p)
+
+                if self.is_dry_run:
+                    provisions.append(
+                        ProvisionProductDryRunTask(**provisioning_parameters)
+                    )
+                else:
+                    provisions.append(
+                        ProvisionProductTask(**provisioning_parameters)
+                    )
+
+            elif task_status == constants.TERMINATED:
+                terminating_parameters = {}
+                for p in TerminateProductTask.get_param_names(include_significant=True):
+                    terminating_parameters[p] = task_def.get(p)
+
+                if self.is_dry_run:
+                    provisions.append(
+                        TerminateProductDryRunTask(**terminating_parameters)
+                    )
+                else:
+                    provisions.append(
+                        TerminateProductTask(**terminating_parameters)
+                    )
+            else:
+                raise Exception(f"Unsupported status of {task_status}")
+        return provisions
+
+    def run(self):
+        launch_tasks_def = manifest_utils_for_launches.generate_launch_task_defs_for_launch(
+            self.launch_name,
+            self.manifest,
+            self.puppet_account_id,
+            self.should_use_sns,
+            self.should_use_product_plans,
+            self.include_expanded_from,
+            self.single_account,
+            self.is_dry_run
+        )
+
+        logger.info(f"{self.uid} starting pre actions")
+        yield [portfoliomanagement.ProvisionActionTask(**p) for p in launch_tasks_def.get('pre_actions', [])]
+        logger.info(f"{self.uid} finished pre actions")
+
+        logger.info(f"{self.uid} starting launches")
+        ls = self.generate_provisions(launch_tasks_def.get('task_defs', []))
+        yield ls
+        logger.info(f"{self.uid} finished launches")
+
+        logger.info(f"{self.uid} starting post actions")
+        yield [portfoliomanagement.ProvisionActionTask(**p) for p in launch_tasks_def.get('post_actions', [])]
+        logger.info(f"{self.uid} finished post actions")
+
+        self.write_output(self.params_for_results_display())
+
+
+class SpokeLocalPortfolioTask(tasks.PuppetTask):
+    spoke_local_portfolio_name = luigi.Parameter()
+    manifest = luigi.DictParameter()
+    puppet_account_id = luigi.Parameter()
+    should_use_sns = luigi.BoolParameter()
+    should_use_product_plans = luigi.BoolParameter()
+    include_expanded_from = luigi.BoolParameter()
+    single_account = luigi.Parameter()
+    is_dry_run = luigi.BoolParameter()
+
+    def params_for_results_display(self):
+        return {
+            "spoke_local_portfolio_name": self.spoke_local_portfolio_name,
+        }
+
+    def requires(self):
+        spoke_local_portfolio = self.manifest.get('spoke-local-portfolios').get(self.spoke_local_portfolio_name)
+
+        return {
+            'other_launches': [
+                LaunchTask(
+                    launch_name=dependency,
+                    manifest=self.manifest,
+                    puppet_account_id=self.puppet_account_id,
+                    should_use_sns=self.should_use_sns,
+                    should_use_product_plans=self.should_use_product_plans,
+                    include_expanded_from=self.include_expanded_from,
+                    single_account=self.single_account,
+                    is_dry_run=self.is_dry_run,
+                ) for dependency in spoke_local_portfolio.get('depends_on', [])
+            ]
+        }
+
+    def generate_tasks(self, task_defs):
+        if len(task_defs) == 0:
+            raise Exception("The configuration for this share does not include any target accounts")
+        first_task_def = task_defs[0]
+        logger.info('first_task_def')
+        portfolio = first_task_def.get('portfolio')
+        tasks = []
+
+        for task_def in task_defs:
+            create_spoke_local_portfolio_task_params = {
+                'account_id': task_def.get('account_id'),
+                'region': task_def.get('region'),
+                'portfolio': portfolio,
+                'organization': task_def.get('organization')
+            }
+            create_spoke_local_portfolio_task = portfoliomanagement.CreateSpokeLocalPortfolioTask(
+                **create_spoke_local_portfolio_task_params,
+            )
+            tasks.append(create_spoke_local_portfolio_task)
+
+            create_spoke_local_portfolio_task_as_dependency_params = {
+                'account_id': task_def.get('account_id'),
+                'region': task_def.get('region'),
+                'portfolio': portfolio,
+                'organization': task_def.get('organization'),
+            }
+
+            if len(task_def.get('associations', [])) > 0:
+                create_associations_for_portfolio_task = portfoliomanagement.CreateAssociationsForPortfolioTask(
+                    **create_spoke_local_portfolio_task_as_dependency_params,
+                    associations=task_def.get('associations'),
+                    puppet_account_id=task_def.get('puppet_account_id'),
+                    should_use_sns=task_def.get('should_use_sns'),
+                    # dependencies=dependencies,
+                    # pre_actions=task_def.get('pre_actions'),
+                )
+                tasks.append(create_associations_for_portfolio_task)
+
+            launch_constraints = task_def.get('constraints', {}).get('launch', [])
+
+            import_into_spoke_local_portfolio_task = portfoliomanagement.ImportIntoSpokeLocalPortfolioTask(
+                **create_spoke_local_portfolio_task_as_dependency_params,
+                puppet_account_id=task_def.get('puppet_account_id'),
+            )
+            tasks.append(import_into_spoke_local_portfolio_task)
+
+            if len(launch_constraints) > 0:
+                create_launch_role_constraints_for_portfolio_task_params = {
+                    'launch_constraints': launch_constraints,
+                    'puppet_account_id': task_def.get('puppet_account_id'),
+                    'should_use_sns': task_def.get('should_use_sns'),
+                }
+                create_launch_role_constraints_for_portfolio = portfoliomanagement.CreateLaunchRoleConstraintsForPortfolio(
+                    **create_spoke_local_portfolio_task_as_dependency_params,
+                    **create_launch_role_constraints_for_portfolio_task_params,
+                    # dependencies=dependencies,
+                )
+                tasks.append(create_launch_role_constraints_for_portfolio)
+        return tasks
+
+    def run(self):
+        self.info("started")
+        tasks_defs = manifest_utils_for_spoke_local_portfolios.generate_spoke_local_portfolios_tasks_for_spoke_local_portfolio(
+            self.spoke_local_portfolio_name,
+            self.manifest,
+            self.puppet_account_id,
+            self.should_use_sns,
+            self.should_use_product_plans,
+            self.include_expanded_from,
+            self.single_account,
+            self.is_dry_run,
+        )
+        self.info(json.dumps(tasks_defs, default=str))
+
+        logger.info(f"{self.uid} starting pre actions")
+        yield [portfoliomanagement.ProvisionActionTask(**p) for p in tasks_defs.get('pre_actions', [])]
+        logger.info(f"{self.uid} finished pre actions")
+
+        logger.info(f"{self.uid} starting launches")
+        yield self.generate_tasks(tasks_defs.get('task_defs', []))
+        logger.info(f"{self.uid} finished launches")
+
+        logger.info(f"{self.uid} starting post actions")
+        yield [portfoliomanagement.ProvisionActionTask(**p) for p in tasks_defs.get('post_actions', [])]
+        logger.info(f"{self.uid} finished post actions")
+
+        self.write_output(self.params_for_results_display())
+        self.info("finished")
